@@ -55,6 +55,8 @@
 #'                 all available threads will be used; c) a number strictly
 #'                 between 0 and 1 which represents the fraction of all threads 
 #'                 to use. The default is to use 1 core.
+#' @param ssc An object of class `boot_ssc.type` obtained with the function \code{\link[fwildclusterboot]{boot_ssc}}. Represents how the small sample adjustments are computed. The defaults are `adj = TRUE, fixef.K = "none", cluster.adj = "TRUE", cluster.df = "conventional"`. 
+#'             You can find more details in the help file for `boot_ssc()`. The function is purposefully designed to mimic fixest's \code{\link[fixest]{ssc}} function. 
 #' @param ... Further arguments passed to or from other methods.
 #'
 #' @importFrom dreamerr check_arg validate_dots
@@ -70,9 +72,13 @@
 #' \item{N_G}{Dimension of the cluster variables as used in boottest.}
 #' \item{sign_level}{Significance level used in boottest.}
 #' \item{type}{Distribution of the bootstrap weights.}
+#' \item{impose_null}{Whether the null was imposed on the bootstrap dgp or not.}
+#' \item{R}{The vector "R" in the null hypothesis of interest Rbeta = beta0.}
+#' \item{beta0}{The scalar "beta0" in the null hypothesis of interest Rbeta = beta0.}
+#' \item{point_estimate}{R'beta. A scalar: the constraints vector times the regression coefficients.}
 #' \item{p_test_vals}{All p-values calculated while calculating the confidence
 #'      interval.}
-#' \item{t_stat}{The original test statistics - either imposing the null or not - with small sample correction `G / (G-1)`.}
+#' \item{t_stat}{The 'original' regression test statistics.}
 #' \item{test_vals}{All t-statistics calculated while calculating the 
 #'       confidence interval.}
 #'  \item{t_boot}{All bootstrap t-statistics.}     
@@ -166,6 +172,11 @@ boottest.felm <- function(object,
                           maxiter = 10,
                           na_omit = TRUE,
                           nthreads = getBoottest_nthreads(), 
+                          ssc = boot_ssc(adj = TRUE, 
+                                         fixef.K = "none", 
+                                         cluster.adj = TRUE, 
+                                         cluster.df = "conventional"),
+                          #fweights = FALSE,
                           ...) {
 
   call <- match.call()
@@ -184,6 +195,9 @@ boottest.felm <- function(object,
   check_arg(bootcluster, "character vector")
   check_arg(tol, "numeric scalar")
   check_arg(maxiter, "scalar integer")
+  check_arg(boot_ssc, "class(boot_ssc.type)")
+  #check_arg(fweights, "logical scalar")
+  
   
   # check appropriateness and assign nthreads
   nthreads <- check_set_nthreads(nthreads)
@@ -283,7 +297,18 @@ boottest.felm <- function(object,
                             param = param,
                             bootcluster = bootcluster, 
                             na_omit = na_omit, 
-                            R = R)
+                            R = R#, 
+                            #fweights = fweights
+                            )
+  
+  N <- preprocess$N
+  k <- length(coef(object))
+  G <- vapply(preprocess$clustid, function(x) length(unique(x)), numeric(1))
+  vcov_sign <- preprocess$vcov_sign
+  
+  
+  small_sample_correction <- get_ssc(boot_ssc_object = ssc, N = N, k = k, G = G, vcov_sign = vcov_sign)
+  
 
   clustid_dims <- preprocess$clustid_dims
   # Invert p-value
@@ -320,7 +345,8 @@ boottest.felm <- function(object,
     p_val_type = p_val_type, 
     nthreads = nthreads, 
     type = type, 
-    full_enumeration = full_enumeration
+    full_enumeration = full_enumeration, 
+    small_sample_correction = small_sample_correction
   )
 
 
