@@ -51,6 +51,8 @@
 #'                 all available threads will be used; c) a number strictly
 #'                 between 0 and 1 which represents the fraction of all threads 
 #'                 to use. The default is to use 1 core.
+#' @param ssc An object of class `boot_ssc.type` obtained with the function \code{\link[fwildclusterboot]{boot_ssc}}. Represents how the small sample adjustments are computed. The defaults are `adj = TRUE, fixef.K = "none", cluster.adj = "TRUE", cluster.df = "conventional"`. 
+#'             You can find more details in the help file for `boot_ssc()`. The function is purposefully designed to mimic fixest's \code{\link[fixest]{ssc}} function. 
 #' @param ... Further arguments passed to or from other methods.
 #' 
 #' @importFrom dreamerr check_arg validate_dots
@@ -62,20 +64,25 @@
 #' \item{p_val}{The bootstrap p-value.}
 #' \item{conf_int}{The bootstrap confidence interval.}
 #' \item{param}{The tested parameter.}
-#' \item{N}{Sample size. Might differ from the regression sample size if the 
-#'          cluster variables contain NA values.}
+#' \item{N}{Sample size. Might differ from the regression sample size if 
+#'      the cluster variables contain NA values.}
 #' \item{B}{Number of Bootstrap Iterations.}
 #' \item{clustid}{Names of the cluster Variables.}
 #' \item{N_G}{Dimension of the cluster variables as used in boottest.}
 #' \item{sign_level}{Significance level used in boottest.}
 #' \item{type}{Distribution of the bootstrap weights.}
-#' \item{t_stat}{The original test statistics - either imposing the null or not - with small sample correction `G / (G-1)`.}
+#' \item{impose_null}{Whether the null was imposed on the bootstrap dgp or not.}
+#' \item{R}{The vector "R" in the null hypothesis of interest Rbeta = beta0.}
+#' \item{beta0}{The scalar "beta0" in the null hypothesis of interest Rbeta = beta0.}
+#' \item{point_estimate}{R'beta. A scalar: the constraints vector times the regression coefficients.}
+#' \item{p_test_vals}{All p-values calculated while calculating the confidence
+#'      interval.}
+#' \item{t_stat}{The 'original' regression test statistics.}
 #' \item{test_vals}{All t-statistics calculated while calculating the 
 #'       confidence interval.}
 #'  \item{t_boot}{All bootstrap t-statistics.}     
 #' \item{regression}{The regression object used in boottest.}
 #' \item{call}{Function call of boottest.}
-#' 
 #' @export
 #' 
 #' @section Confidence Intervals:
@@ -148,8 +155,14 @@ boottest.lm <- function(object,
                         maxiter = 10,
                         na_omit = TRUE, 
                         nthreads = getBoottest_nthreads(), 
+                        ssc = boot_ssc(adj = TRUE, 
+                                       fixef.K = "none", 
+                                       cluster.adj = TRUE, 
+                                       cluster.df = "conventional"),
+                        #fweights = FALSE,
                         ...) {
 
+
   call <- match.call()
   dreamerr::validate_dots(stop = TRUE)
   
@@ -164,6 +177,8 @@ boottest.lm <- function(object,
   check_arg(bootcluster, "character vector")
   check_arg(tol, "numeric scalar")
   check_arg(maxiter, "scalar integer")
+  check_arg(boot_ssc, "class(boot_ssc.type)")
+  #check_arg(fweights, "logical scalar")
   
   # check appropriateness of nthreads
   nthreads <- check_set_nthreads(nthreads)
@@ -188,7 +203,7 @@ boottest.lm <- function(object,
          call. = FALSE)
   }
   
-  if(p_val_type %in% c(">", "<") && conf_int == TRUE){
+  if(p_val_type %in% c(">", "<") && (is.null(conf_int) || conf_int == TRUE)){
     conf_int <- FALSE
     warning(paste("Currently, boottest() does not calculate confidence intervals for one-sided hypotheses, but this will change in a future release."), call. = FALSE)
   }
@@ -256,10 +271,21 @@ boottest.lm <- function(object,
                             param = param,
                             bootcluster = bootcluster, 
                             na_omit = na_omit, 
-                            R = R)
+                            R = R#, 
+                            #fweights = fweights
+                            )
   
+  N <- preprocess$N
+  k <- length(coef(object))
+  G <- vapply(preprocess$clustid, function(x) length(unique(x)), numeric(1))
+  vcov_sign <- preprocess$vcov_sign
+
   
-  clustid_dims <- preprocess$clustid_dims
+  small_sample_correction <- get_ssc(boot_ssc_object = ssc, N = N, k = k, G = G, vcov_sign = vcov_sign)
+  
+
+  
+  #clustid_dims <- preprocess$clustid_dims
   point_estimate <- as.vector(object$coefficients[param] %*% preprocess$R0[param])
   
   clustid_fml <- as.formula(paste("~", paste(clustid, collapse = "+")))
@@ -282,9 +308,8 @@ boottest.lm <- function(object,
     full_enumeration <- FALSE
   }
   
-  
   # conduct inference: calculate p-value
-  res <- boot_algo2(preprocess,
+  res <- boot_algo2(preprocessed_object = preprocess,
                     boot_iter = B,
                     point_estimate = point_estimate,
                     impose_null = impose_null,
@@ -295,12 +320,16 @@ boottest.lm <- function(object,
                     p_val_type = p_val_type, 
                     nthreads = nthreads, 
                     type = type, 
-                    full_enumeration = full_enumeration
+                    full_enumeration = full_enumeration, 
+                    small_sample_correction = small_sample_correction
   )
   
+  if(!is.null(res$invalid_t)){
+    message(paste0(res$invalid_t), "test statistics deleted due to non-positive definite covariance matrix.")
+  }
+
   # compute confidence sets
   
-  
   if (is.null(conf_int) || conf_int == TRUE) {
     
     # guess for standard errors
@@ -311,10 +340,7 @@ boottest.lm <- function(object,
       se_guess <- abs((point_estimate - beta0) / res$t_stat)
     }
     
-    # if (is.na(se_guess)) {
-    #   se_guess <- object$se[param]
-    # }
-    
+
     res_p_val <- invert_p_val(
       object = res,
       boot_iter = B,
@@ -338,7 +364,7 @@ boottest.lm <- function(object,
   
   res_final <- list(
     point_estimate = point_estimate,
-    p_val = res[["p_val"]],
+    p_val = res$p_val,
     conf_int = res_p_val$conf_int,
     p_test_vals = res_p_val$p_grid_vals,
     test_vals = res_p_val$grid_vals,
