@@ -3,7 +3,7 @@
 # Tests A1 and A2
 # ------------------------------------------------------------------ # 
 
-# library(fwildclusterboot)
+library(fwildclusterboot)
 
 
 # ---------------------------------------------------------------------------------------------- # 
